package com.sun.jmx.mbeanserver;

import javax.management.MBeanPermission;
import javax.management.loading.PrivateClassLoader;
import sun.reflect.misc.ReflectUtil;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;
import javax.management.ObjectName;
import java.util.List;
import java.util.Map;

final class ClassLoaderRepositorySupport implements ModifiableClassLoaderRepository
{
    private static final ClassLoaderRepositorySupport.LoaderEntry[] EMPTY_LOADER_ARRAY;
    private ClassLoaderRepositorySupport.LoaderEntry[] loaders;
    private final Map<String, List<ClassLoader>> search;
    private final Map<ObjectName, ClassLoader> loadersWithNames;
    
    ClassLoaderRepositorySupport() {
        this.loaders = ClassLoaderRepositorySupport.EMPTY_LOADER_ARRAY;
        this.search = new Hashtable(10);
        this.loadersWithNames = new Hashtable(10);
    }
    
    private synchronized boolean add(ObjectName objectName, ClassLoader classLoader) {
        ArrayList list = new ArrayList(Arrays.asList(this.loaders));
        list.add(new ClassLoaderRepositorySupport.LoaderEntry(objectName, classLoader));
        this.loaders = (ClassLoaderRepositorySupport.LoaderEntry[])list.toArray(ClassLoaderRepositorySupport.EMPTY_LOADER_ARRAY);
        return true;
    }
    
    private synchronized boolean remove(ObjectName objectName, ClassLoader classLoader) {
        int length = this.loaders.length;
        int i = 0;
        while (i < length) {
            ClassLoaderRepositorySupport.LoaderEntry loaderEntry = this.loaders[i];
            if (((objectName == null) ? (classLoader == loaderEntry.loader) : objectName.equals(loaderEntry.name))) {
                ClassLoaderRepositorySupport.LoaderEntry[] loaders = new ClassLoaderRepositorySupport.LoaderEntry[length - 1];
                System.arraycopy(this.loaders, 0, loaders, 0, i);
                System.arraycopy(this.loaders, i + 1, loaders, i, length - 1 - i);
                this.loaders = loaders;
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public final Class<?> loadClass(String s) throws ClassNotFoundException {
        return this.loadClass(this.loaders, s, null, null);
    }
    
    @Override
    public final Class<?> loadClassWithout(ClassLoader obj, String str) throws ClassNotFoundException {
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, ClassLoaderRepositorySupport.class.getName(), "loadClassWithout", str + " without " + obj);
        }
        if (obj == null) {
            return this.loadClass(this.loaders, str, null, null);
        }
        this.startValidSearch(obj, str);
        try {
            return this.loadClass(this.loaders, str, obj, null);
        }
        finally {
            this.stopValidSearch(obj, str);
        }
    }
    
    @Override
    public final Class<?> loadClassBefore(ClassLoader obj, String str) throws ClassNotFoundException {
        if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, ClassLoaderRepositorySupport.class.getName(), "loadClassBefore", str + " before " + obj);
        }
        if (obj == null) {
            return this.loadClass(this.loaders, str, null, null);
        }
        this.startValidSearch(obj, str);
        try {
            return this.loadClass(this.loaders, str, null, obj);
        }
        finally {
            this.stopValidSearch(obj, str);
        }
    }
    
    private Class<?> loadClass(ClassLoaderRepositorySupport.LoaderEntry[] array, String s, ClassLoader classLoader, ClassLoader classLoader2) throws ClassNotFoundException {
        ReflectUtil.checkPackageAccess(s);
        for (int length = array.length, i = 0; i < length; ++i) {
            try {
                ClassLoader loader = array[i].loader;
                if (loader == null) {
                    return Class.forName(s, false, null);
                }
                if (loader != classLoader) {
                    if (loader == classLoader2) {
                        break;
                    }
                    if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
                        JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, ClassLoaderRepositorySupport.class.getName(), "loadClass", "Trying loader = " + loader);
                    }
                    return Class.forName(s, false, loader);
                }
            }
            catch (ClassNotFoundException ex) {}
        }
        throw new ClassNotFoundException(s);
    }
    
    private synchronized void startValidSearch(ClassLoader classLoader, String s) throws ClassNotFoundException {
        List<ClassLoader> list = (List)this.search.get(s);
        if (list != null && list.contains(classLoader)) {
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, ClassLoaderRepositorySupport.class.getName(), "startValidSearch", "Already requested loader = " + classLoader + " class = " + s);
            }
            throw new ClassNotFoundException(s);
        }
        else {
            if (list == null) {
                list = new ArrayList(1);
                this.search.put(s, list);
            }
            list.add(classLoader);
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, ClassLoaderRepositorySupport.class.getName(), "startValidSearch", "loader = " + classLoader + " class = " + s);
            }
            return;
        }
    }
    
    private synchronized void stopValidSearch(ClassLoader obj, String str) {
        List<Object> list = (List)this.search.get(str);
        if (list != null) {
            list.remove(obj);
            if (JmxProperties.MBEANSERVER_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.MBEANSERVER_LOGGER.logp(Level.FINER, ClassLoaderRepositorySupport.class.getName(), "stopValidSearch", "loader = " + obj + " class = " + str);
            }
        }
    }
    
    @Override
    public final void addClassLoader(ClassLoader classLoader) {
        this.add(null, classLoader);
    }
    
    @Override
    public final void removeClassLoader(ClassLoader classLoader) {
        this.remove(null, classLoader);
    }
    
    @Override
    public final synchronized void addClassLoader(ObjectName objectName, ClassLoader classLoader) {
        this.loadersWithNames.put(objectName, classLoader);
        if (!(classLoader instanceof PrivateClassLoader)) {
            this.add(objectName, classLoader);
        }
    }
    
    @Override
    public final synchronized void removeClassLoader(ObjectName objectName) {
        ClassLoader classLoader = (ClassLoader)this.loadersWithNames.remove(objectName);
        if (!(classLoader instanceof PrivateClassLoader)) {
            this.remove(objectName, classLoader);
        }
    }
    
    @Override
    public final ClassLoader getClassLoader(ObjectName objectName) {
        ClassLoader classLoader = (ClassLoader)this.loadersWithNames.get(objectName);
        if (classLoader != null) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPermission(new MBeanPermission(classLoader.getClass().getName(), null, objectName, "getClassLoader"));
            }
        }
        return classLoader;
    }
    
    static {
        EMPTY_LOADER_ARRAY = new ClassLoaderRepositorySupport.LoaderEntry[0];
    }
    
    private static class LoaderEntry
    {
        ObjectName name;
        ClassLoader loader;
        
        LoaderEntry(ObjectName name, ClassLoader loader) {
            this.name = name;
            this.loader = loader;
        }
    }
}
